import { EditorState, Transaction, StateCommand, Facet, Extension } from '@codemirror/state';
import { EditorView, KeyBinding, Command } from '@codemirror/view';
import { NodeType } from 'lezer-tree';

interface CompletionConfig {
    activateOnTyping?: boolean;
    override?: readonly CompletionSource[] | null;
    maxRenderedOptions?: number;
    defaultKeymap?: boolean;
}

interface Completion {
    label: string;
    detail?: string;
    info?: string | ((completion: Completion) => Node);
    apply?: string | ((view: EditorView, completion: Completion, from: number, to: number) => void);
    type?: string;
    boost?: number;
}
declare class CompletionContext {
    readonly state: EditorState;
    readonly pos: number;
    readonly explicit: boolean;
    constructor(state: EditorState, pos: number, explicit: boolean);
    tokenBefore(types: readonly string[]): {
        from: number;
        to: number;
        text: string;
        type: NodeType;
    } | null;
    matchBefore(expr: RegExp): {
        from: number;
        to: number;
        text: string;
    } | null;
    get aborted(): boolean;
    addEventListener(type: "abort", listener: () => void): void;
}
declare function completeFromList(list: readonly (string | Completion)[]): CompletionSource;
declare function ifNotIn(nodes: readonly string[], source: CompletionSource): CompletionSource;
declare type CompletionSource = (context: CompletionContext) => CompletionResult | null | Promise<CompletionResult | null>;
interface CompletionResult {
    from: number;
    to?: number;
    options: readonly Completion[];
    span?: RegExp;
}

declare function snippet(template: string): (editor: {
    state: EditorState;
    dispatch: (tr: Transaction) => void;
}, _completion: Completion, from: number, to: number) => void;
declare const clearSnippet: StateCommand;
declare const nextSnippetField: StateCommand;
declare const prevSnippetField: StateCommand;
declare const snippetKeymap: Facet<readonly KeyBinding[], readonly KeyBinding[]>;
declare function snippetCompletion(template: string, completion: Completion): Completion;

declare function moveCompletionSelection(forward: boolean, by?: "option" | "page"): Command;
declare const acceptCompletion: Command;
declare const startCompletion: Command;
declare const closeCompletion: Command;

declare const completeAnyWord: CompletionSource;

declare function autocompletion(config?: CompletionConfig): Extension;
declare const completionKeymap: readonly KeyBinding[];
declare function completionStatus(state: EditorState): null | "active" | "pending";
declare function currentCompletions(state: EditorState): readonly Completion[];

export { Completion, CompletionContext, CompletionResult, CompletionSource, acceptCompletion, autocompletion, clearSnippet, closeCompletion, completeAnyWord, completeFromList, completionKeymap, completionStatus, currentCompletions, ifNotIn, moveCompletionSelection, nextSnippetField, prevSnippetField, snippet, snippetCompletion, snippetKeymap, startCompletion };
